@@ -30,6 +30,7 @@
 #include <config.h>
 #endif
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,7 +39,7 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
+#include <libxfce4panel/libxfce4panel.h>
 
 #ifdef HAVE_UPOWER_GLIB
 #include <upower.h>
@@ -131,7 +132,40 @@ click_event(GtkWidget *w, GdkEventButton *event, t_global_monitor *global)
     return FALSE;
 }
 
-static gint
+static void
+set_fraction(GtkProgressBar *bar, gdouble fraction)
+{
+    GtkAllocation alloc;
+    gint max_alloc;
+
+    /*
+     * Try to avoid a call to GTK's bar_set_fraction() if the new fraction
+     * isn't changing the number of pixels (height or width of the bar) already
+     * displayed on the screen.
+     *
+     * max_alloc is independent from horizontal/vertical orientation of the bar.
+     */
+
+    gtk_widget_get_allocation(GTK_WIDGET(bar), &alloc);
+
+    max_alloc = MAX(alloc.width, alloc.height);
+    if (max_alloc > 0)
+        fraction = round(fraction * max_alloc) / max_alloc;
+
+    if (gtk_progress_bar_get_fraction(bar) != fraction)
+        gtk_progress_bar_set_fraction(bar, fraction);
+}
+
+static void
+set_tooltip(GtkWidget *w, const gchar *caption)
+{
+    gchar *displayed_caption = gtk_widget_get_tooltip_text(w);
+    if (g_strcmp0(displayed_caption, caption) != 0)
+        gtk_widget_set_tooltip_text(w, caption);
+    g_free(displayed_caption);
+}
+
+static void
 update_monitors(t_global_monitor *global)
 {
 
@@ -169,7 +203,7 @@ update_monitors(t_global_monitor *global)
             global->monitor[count]->history[1] =
                 global->monitor[count]->history[0];
 
-            gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(global->monitor[count]->status),
+            set_fraction(GTK_PROGRESS_BAR(global->monitor[count]->status),
                  global->monitor[count]->value_read / 100.0);
         }
     }
@@ -177,14 +211,14 @@ update_monitors(t_global_monitor *global)
     {
         g_snprintf(caption, sizeof(caption), _("System Load: %ld%%"),
                    global->monitor[0]->value_read);
-        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[0]->ebox), caption);
+        set_tooltip(global->monitor[0]->ebox, caption);
     }
 
     if (global->monitor[1]->options.enabled)
     {
         g_snprintf(caption, sizeof(caption), _("Memory: %ldMB of %ldMB used"),
                    MUsed >> 10 , MTotal >> 10);
-        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[1]->ebox), caption);
+        set_tooltip(global->monitor[1]->ebox, caption);
     }
 
     if (global->monitor[2]->options.enabled)
@@ -195,7 +229,7 @@ update_monitors(t_global_monitor *global)
         else
             g_snprintf(caption, sizeof(caption), _("No swap"));
 
-        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[2]->ebox), caption);
+        set_tooltip(global->monitor[2]->ebox, caption);
     }
 
     if (global->uptime->enabled)
@@ -218,9 +252,8 @@ update_monitors(t_global_monitor *global)
                                caption);
             g_snprintf(caption, sizeof(caption), _("Uptime: %d:%02d"), hours, mins);
         }
-        gtk_widget_set_tooltip_text(GTK_WIDGET(global->uptime->ebox), caption);
+        set_tooltip(global->uptime->ebox, caption);
     }
-    return TRUE;
 }
 
 static void
@@ -400,6 +433,15 @@ monitor_free(XfcePanelPlugin *plugin, t_global_monitor *global)
     g_free(global);
 }
 
+static gboolean
+update_monitors_cb(gpointer user_data)
+{
+    t_global_monitor *global = user_data;
+
+    update_monitors (global);
+    return TRUE;
+}
+
 static void
 setup_timer(t_global_monitor *global)
 {
@@ -412,7 +454,7 @@ setup_timer(t_global_monitor *global)
             if (!up_client_get_lid_is_closed(global->upower)) {
                 global->timeout_id = g_timeout_add_seconds(
                                         global->timeout_seconds,
-                                        (GSourceFunc)update_monitors, global);
+                                        update_monitors_cb, global);
             } else {
                 /* Don't do any timeout if the lid is closed on battery */
                 global->timeout_id = 0;
@@ -421,7 +463,7 @@ setup_timer(t_global_monitor *global)
         }
     }
 #endif
-    global->timeout_id = g_timeout_add(global->timeout, (GSourceFunc)update_monitors, global);
+    global->timeout_id = g_timeout_add(global->timeout, update_monitors_cb, global);
     /* reduce the default tooltip timeout to be smaller than the update interval otherwise
      * we won't see tooltips on GTK 2.16 or newer */
     settings = gtk_settings_get_default();
@@ -679,33 +721,26 @@ upower_changed_cb(UpClient *client, t_global_monitor *global)
 static void
 entry_changed_cb(GtkEntry *entry, t_global_monitor *global)
 {
-    gchar** charvar = (gchar**)g_object_get_data (G_OBJECT(entry), "charvar");
-    gboolean** use_label = (gboolean**)g_object_get_data (G_OBJECT(entry), "boolvar");
-    g_free(*charvar);
-    if (gtk_entry_get_text_length (entry) == 0)
-        *use_label = (gboolean *) FALSE;
-    else
-        *use_label = (gboolean *) TRUE;
-    *charvar = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
-
-    setup_monitor(global);
+    gchar **charvar = (gchar**) g_object_get_data (G_OBJECT (entry), "charvar");
+    gboolean *use_label = (gboolean*) g_object_get_data (G_OBJECT (entry), "boolvar");
+    g_free (*charvar);
+    *use_label = (gtk_entry_get_text_length (entry) != 0);
+    *charvar = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+    setup_monitor (global);
 }
 
 static void
 switch_cb(GtkSwitch *check_button, gboolean state, t_global_monitor *global)
 {
-    gboolean oldstate;
-    gboolean* boolvar;
-    gpointer sensitive_widget;
-    boolvar = (gboolean*)g_object_get_data(G_OBJECT(check_button), "boolvar");
-    sensitive_widget = g_object_get_data(G_OBJECT(check_button), "sensitive_widget");
-    oldstate = *boolvar;
+    gboolean *boolvar = (gboolean*) g_object_get_data (G_OBJECT (check_button), "boolvar");
+    gpointer sensitive_widget = g_object_get_data (G_OBJECT (check_button), "sensitive_widget");
+    gboolean oldstate = *boolvar;
     *boolvar = state;
     gtk_switch_set_state (check_button, state);
     if (sensitive_widget)
-        gtk_revealer_set_reveal_child (GTK_REVEALER (sensitive_widget), *boolvar);
-    if (oldstate != *boolvar)
-        setup_monitor(global);
+        gtk_revealer_set_reveal_child (GTK_REVEALER (sensitive_widget), state);
+    if (oldstate != state)
+        setup_monitor (global);
 }
 
 static void
@@ -738,15 +773,10 @@ change_timeout_cb(GtkSpinButton *spin, t_global_monitor *global)
 static void
 change_timeout_seconds_cb(GtkSpinButton *spin, t_global_monitor *global)
 {
-    gboolean** use_timeout_seconds = (gboolean**)g_object_get_data (G_OBJECT(spin), "boolvar");
-    global->timeout_seconds = gtk_spin_button_get_value(spin);
-
-    if (global->timeout_seconds == 0)
-        *use_timeout_seconds = (gboolean *) FALSE;
-    else
-        *use_timeout_seconds = (gboolean *) TRUE;
-
-    setup_timer(global);
+    gboolean *use_timeout_seconds = (gboolean*) g_object_get_data (G_OBJECT (spin), "boolvar");
+    global->timeout_seconds = gtk_spin_button_get_value (spin);
+    *use_timeout_seconds = (global->timeout_seconds != 0);
+    setup_timer (global);
 }
 #endif
 
@@ -767,11 +797,12 @@ static GtkWidget *new_label (GtkGrid *grid, guint row,
 }
 
 /* Create a new monitor setting  with gtkswitch, and eventually a color button and a checkbox + entry */
-static void new_monitor_setting(t_global_monitor *global, GtkGrid *grid, int position,
-                           const gchar *title, gboolean *boolvar, GdkRGBA* colorvar,
-                           gboolean * use_label, gchar **labeltext)
+static void
+new_monitor_setting (t_global_monitor *global, GtkGrid *grid, int position,
+                     const gchar *title, gboolean *boolvar, GdkRGBA *colorvar,
+                     gboolean *use_label, gchar **labeltext)
 {
-    GtkWidget *revealer, *subgrid, *sw, *label, *button, *entry;
+    GtkWidget *sw, *label;
     gchar *markup;
 
     sw = gtk_switch_new();
@@ -780,7 +811,6 @@ static void new_monitor_setting(t_global_monitor *global, GtkGrid *grid, int pos
     gtk_widget_set_halign (sw, GTK_ALIGN_END);
     gtk_widget_set_valign (sw, GTK_ALIGN_CENTER);
     gtk_widget_set_margin_top (sw, 12);
-    switch_cb (GTK_SWITCH(sw), *boolvar, global);
     g_signal_connect (GTK_WIDGET(sw), "state-set",
                       G_CALLBACK(switch_cb), global);
 
@@ -794,42 +824,48 @@ static void new_monitor_setting(t_global_monitor *global, GtkGrid *grid, int pos
     gtk_grid_attach(GTK_GRID(grid), label, 0, position, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), sw, 1, position, 1, 1);
 
-    if (colorvar == NULL)
-        return;
-    revealer = gtk_revealer_new ();
-    subgrid = gtk_grid_new ();
-    gtk_container_add (GTK_CONTAINER (revealer), subgrid);
-    gtk_revealer_set_reveal_child (GTK_REVEALER (revealer), TRUE);
-    g_object_set_data (G_OBJECT(sw), "sensitive_widget", revealer);
-    gtk_grid_attach(GTK_GRID(grid), revealer, 0, position + 1, 2, 1);
-    gtk_grid_set_column_spacing (GTK_GRID(subgrid), 12);
-    gtk_grid_set_row_spacing (GTK_GRID(subgrid), 6);
+    if (colorvar != NULL)
+    {
+        GtkWidget *revealer, *subgrid, *button, *entry;
 
-    label = gtk_label_new_with_mnemonic (_("Options:"));
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
-    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
-    gtk_widget_set_margin_start (label, 12);
-    gtk_grid_attach (GTK_GRID(subgrid), label, 0, 0, 1, 1);
+        revealer = gtk_revealer_new ();
+        subgrid = gtk_grid_new ();
+        gtk_container_add (GTK_CONTAINER (revealer), subgrid);
+        gtk_revealer_set_reveal_child (GTK_REVEALER (revealer), TRUE);
+        g_object_set_data (G_OBJECT(sw), "sensitive_widget", revealer);
+        gtk_grid_attach(GTK_GRID(grid), revealer, 0, position + 1, 2, 1);
+        gtk_grid_set_column_spacing (GTK_GRID(subgrid), 12);
+        gtk_grid_set_row_spacing (GTK_GRID(subgrid), 6);
 
-    /* Entry for the optional monitor label */
-    entry = gtk_entry_new ();
-    gtk_widget_set_hexpand (entry, TRUE);
-    gtk_widget_set_margin_start (entry, 12);
-    g_object_set_data (G_OBJECT(entry), "charvar", labeltext);
-    g_object_set_data (G_OBJECT(entry), "boolvar", use_label);
-    gtk_entry_set_text (GTK_ENTRY(entry), *labeltext);
-    g_signal_connect (G_OBJECT(entry), "changed",
-    G_CALLBACK(entry_changed_cb), global);
-    gtk_grid_attach(GTK_GRID(subgrid), entry, 1, 0, 1, 1);
+        label = gtk_label_new_with_mnemonic (_("Options:"));
+        gtk_widget_set_halign (label, GTK_ALIGN_START);
+        gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+        gtk_widget_set_margin_start (label, 12);
+        gtk_grid_attach (GTK_GRID(subgrid), label, 0, 0, 1, 1);
 
-    /* Colorbutton to set the progressbar color */
-    button = gtk_color_button_new_with_rgba(colorvar);
-    gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-    gtk_widget_set_halign(button, GTK_ALIGN_START);
-    g_object_set_data(G_OBJECT(button), "colorvar", colorvar);
-    g_signal_connect(G_OBJECT(button), "color-set",
-                 G_CALLBACK (color_set_cb), global);
-    gtk_grid_attach(GTK_GRID(subgrid), button, 2, 0, 1, 1);
+        /* Entry for the optional monitor label */
+        entry = gtk_entry_new ();
+        gtk_widget_set_hexpand (entry, TRUE);
+        gtk_widget_set_margin_start (entry, 12);
+        g_object_set_data (G_OBJECT(entry), "charvar", labeltext);
+        g_object_set_data (G_OBJECT(entry), "boolvar", use_label);
+        if (*use_label)
+            gtk_entry_set_text (GTK_ENTRY (entry), *labeltext);
+        g_signal_connect (G_OBJECT(entry), "changed",
+        G_CALLBACK(entry_changed_cb), global);
+        gtk_grid_attach(GTK_GRID(subgrid), entry, 1, 0, 1, 1);
+
+        /* Colorbutton to set the progressbar color */
+        button = gtk_color_button_new_with_rgba(colorvar);
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+        gtk_widget_set_halign(button, GTK_ALIGN_START);
+        g_object_set_data(G_OBJECT(button), "colorvar", colorvar);
+        g_signal_connect(G_OBJECT(button), "color-set",
+                     G_CALLBACK (color_set_cb), global);
+        gtk_grid_attach(GTK_GRID(subgrid), button, 2, 0, 1, 1);
+    }
+
+    switch_cb (GTK_SWITCH (sw), *boolvar, global);
 }
 
 static void
@@ -839,7 +875,6 @@ monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
     GtkBox              *content;
     GtkWidget           *grid, *label, *entry, *button;
     guint                count;
-    t_monitor           *monitor;
     static const gchar *FRAME_TEXT[] = {
             N_ ("CPU monitor"),
             N_ ("Memory monitor"),
@@ -911,7 +946,7 @@ monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
     /* Add options for the three monitors */
     for(count = 0; count < 3; count++)
     {
-        monitor = global->monitor[count];
+        t_monitor *monitor = global->monitor[count];
 
         new_monitor_setting(global, GTK_GRID(grid), 4 + 2 * count,
                            _(FRAME_TEXT[count]),
@@ -945,8 +980,8 @@ monitor_show_about(XfcePanelPlugin *plugin, t_global_monitor *global)
       "version", PACKAGE_VERSION,
       "program-name", PACKAGE_NAME,
       "comments", _("Monitor CPU load, swap usage and memory footprint"),
-      "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-systemload-plugin",
-      "copyright", _("Copyright (c) 2003-2018\n"),
+      "website", "https://docs.xfce.org/panel-plugins/xfce4-systemload-plugin",
+      "copyright", _("Copyright (c) 2003-2020\n"),
       "authors", auth, NULL);
 
    if(icon)
